"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var handler_exports = {};
__export(handler_exports, {
  makeHandler: () => makeHandler
});
module.exports = __toCommonJS(handler_exports);
var import_combine_fragments = require("../fragments/combine-fragments");
var import_lambda = require("../fragments/lambda");
var import_helpers = require("../helpers");
var import_handler = require("../stacks/handler");
function makeHandler(config, model, cdc, template) {
  const {
    table: { tableName }
  } = model;
  const {
    directory,
    event,
    filename,
    functionName,
    memorySize,
    readableTables,
    sourceModelName,
    timeout,
    nestedStackLocation,
    writableTables
  } = cdc;
  (0, import_lambda.writeLambda)(directory, template);
  const stackName = (0, import_handler.makeHandlerStackName)(model, cdc);
  const stack = {
    Properties: {
      Location: nestedStackLocation,
      Parameters: {
        CodeUri: filename,
        DetailType: event === "UPSERT" ? "INSERT,MODIFY" : event,
        MemorySize: memorySize,
        SourceModelName: sourceModelName,
        StageName: { Ref: "StageName" },
        TableArn: { "Fn::GetAtt": [tableName, "Arn"] },
        // This is the table logical name, not the table name
        TableName: tableName,
        Timeout: timeout
      }
    },
    Type: "AWS::Serverless::Application"
  };
  const tableAccessPolicy = {
    Properties: {
      PolicyDocument: {
        Statement: [
          readableTables.length && {
            Action: [
              "dynamodb:GetItem",
              "dynamodb:Scan",
              "dynamodb:Query",
              "dynamodb:BatchGetItem",
              "dynamodb:DescribeTable"
            ],
            Effect: "Allow",
            Resource: [
              ...readableTables.map((name) => ({
                "Fn::GetAtt": [name, "Arn"]
              })),
              ...readableTables.map((name) => ({
                "Fn::Sub": [
                  // eslint-disable-next-line no-template-curly-in-string
                  "${tableArn}/index/*",
                  { tableArn: { "Fn::GetAtt": [name, "Arn"] } }
                ]
              }))
            ]
          },
          writableTables.length && {
            Action: [
              "dynamodb:GetItem",
              "dynamodb:DeleteItem",
              "dynamodb:PutItem",
              "dynamodb:Scan",
              "dynamodb:Query",
              "dynamodb:UpdateItem",
              "dynamodb:BatchWriteItem",
              "dynamodb:BatchGetItem",
              "dynamodb:DescribeTable",
              "dynamodb:ConditionCheckItem"
            ],
            Effect: "Allow",
            Resource: [
              ...writableTables.map((name) => ({
                "Fn::GetAtt": [name, "Arn"]
              })),
              ...writableTables.map((name) => ({
                "Fn::Sub": [
                  // eslint-disable-next-line no-template-curly-in-string
                  "${tableArn}/index/*",
                  { tableArn: { "Fn::GetAtt": [name, "Arn"] } }
                ]
              }))
            ]
          }
        ].filter(import_helpers.filterNull),
        Version: "2012-10-17"
      },
      PolicyName: `${functionName}TableAccessPolicy`,
      Roles: [
        // @ts-expect-error typedef doesn't include intrinsic functions
        { "Fn::GetAtt": [stackName, "Outputs.Role"] }
      ]
    },
    Type: "AWS::IAM::Policy"
  };
  const tableAccessPolicyName = `${functionName}TableAccessPolicy`;
  const fragment = (0, import_combine_fragments.combineFragments)({
    Resources: {
      [stackName]: stack,
      ...readableTables.length || writableTables.length ? { [tableAccessPolicyName]: tableAccessPolicy } : {}
    }
  });
  const handlerStack = (0, import_handler.makeHandlerStack)(config);
  return { fragment, stack: handlerStack };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  makeHandler
});
