import { combineFragments } from "../fragments/combine-fragments";
import { writeLambda } from "../fragments/lambda";
import {
  makeDispatcherStack,
  makeDispatcherStackName
} from "../stacks/dispatcher";
function defineTableCdc(table, config) {
  const { dispatcherConfig, dependenciesModuleId, tableName } = table;
  const {
    batchSize,
    filename,
    directory,
    maximumRetryAttempts,
    memorySize,
    nestedStackLocation,
    runtimeModuleId,
    timeout
  } = dispatcherConfig;
  writeLambda(
    directory,
    `// This file is generated. Do not edit by hand.

import {makeDynamoDBStreamDispatcher} from '${runtimeModuleId}';
import * as dependencies from '${dependenciesModuleId}';

export const handler = makeDynamoDBStreamDispatcher({
  ...dependencies,
  tableName: '${tableName}',
});
`
  );
  const stack = {
    Properties: {
      Location: nestedStackLocation,
      Parameters: {
        BatchSize: batchSize,
        CodeUri: filename,
        EventBus: "default",
        MaximumRetryAttempts: maximumRetryAttempts,
        MemorySize: memorySize,
        StreamArn: { "Fn::GetAtt": [tableName, "StreamArn"] },
        Timeout: timeout
      }
    },
    Type: "AWS::Serverless::Application"
  };
  const stackName = makeDispatcherStackName(table);
  const fragment = combineFragments({
    Resources: {
      [stackName]: stack
    }
  });
  const dispatcherStack = makeDispatcherStack(config);
  return { fragment, stack: dispatcherStack };
}
export {
  defineTableCdc
};
